﻿#!/usr/bin/python
# -*- coding: utf-8 -*-
"""『SS7 Op.Ptyhon実行』活用サンプル 「各ステップの負担せん断力の描画」

このモジュールは『SS7 Op.Ptyhon実行』を活用していただくためのサンプルプログラムです。
指定したフォルダ内にある『SS7』の入力データCSVから、解析ステップごとに物件データを作成し解析し、
解析結果から、指定したブレースの負担せんだん力を取得し、Q-δ図に描画するプログラムとなっています。
このサンプルプログラムにより、『SS7 Op.Ptyhon実行』の使用方法として、
・『SS7』の物件データの基本的な扱い方
・『SS7』にコマンドを送信して動作させる方法
・『SS7』CSVデータの変更方法
を学ぶことができます。
また、複数の『SS7』物件データに対して連続的に何かの処理を行わせたいバッチ処理のための雛形としても活用いただけます。

"""

# SS7 Op.Python実行パッケージ
from Python import Ss7Python as ss7cmd
import csv
import matplotlib.pyplot as plt

# --------------------------------------------------------------
# 設定

# 入力CSVのフルパス
input_csv: str = r"C:\UsrData\SS7CSV\Sample1.csv"
# 出力用フォルダのフルパス
output_folder: str = r"C:\UsrData\SS7CSV\OutData"

# 対象ケース
target_case: str = "case=DSX+"  # ケース

# ステップ数
max_stepno: int = 50  # 最大ステップ数
step_pitch: int = 1  # ステップ間隔

# 応力経路を確認するブレース
floor: str = "4"  # 階
flame: str = "A"  # フレーム
jiku1: str = "1"  # 軸1
jiku2: str = "2"  # 軸2


# --------------------------------------------------------------
# CSVファイルを作成する
def make_csvfile(stepNo: int, input_filename: str, csv_filename: str) -> None:
    """make_csvfile 関数

    解析ステップ数を変更した入力データCSVを新規に作成します。

    Args:
        step_no (int):解析ステップ数
        input_filename(str):入力データCSVファイル名
        csv_filename(str):解析ステップ数を変更した入力データCSVファイル名

    """
    teigi_name: str = "Ds算定時の定義"
    if "Q" in target_case:
        teigi_name = "保有水平耐力時の定義"

    case_no: int = 0  # 0=正加力X加力時
    if "X-" in target_case:
        case_no = 2  # 2=負加力X加力時
    elif "Y+" in target_case:
        case_no = 1  # 1=正加力Y加力時
    elif "Y-" in target_case:
        case_no = 3  # 3=負加力Y加力時

    # 特定の行を識別するキーワード
    keyword: str = "最大ステップ数"

    # CSVファイルを読み込む
    with open(input_filename, "r") as input_file:
        reader = csv.reader(input_file)
        data = list(reader)

    # データを変更する
    taige_name_row: bool = False
    processing_row: bool = False
    for i, row in enumerate(data):
        if teigi_name in row:
            taige_name_row = True
        if taige_name_row:
            if keyword in row:
                processing_row = True
            if processing_row:
                if i + case_no < len(data):
                    next_row = data[i + case_no]
                    next_row[-1] = str(stepNo)
                    break

    # 変更したデータを新しいCSVファイルに書き込む
    with open(csv_filename, "w", newline="") as output_file:
        writer = csv.writer(output_file)
        writer.writerows(data)


# --------------------------------------------------------------
# ブレースの負担せん断力の取得
def get_brace_stress(output_filename: str) -> float:
    """get_brace_stress 関数

    設定で指定したブレースの負担せん断力を取得します。

    Args:
        output_csv_path(str):結果データCSVファイルのパス

    Returns:
        float:指定したブレースの負担せん断力

    """
    # 特定の行を識別するキーワード
    keyword: str = "<data>"

    # CSVファイルを読み込む
    with open(output_filename, "r") as output_file:
        reader = csv.reader(output_file)
        data = list(reader)

    # データを検索する
    case_row = False
    processing_row = False
    for i, row in enumerate(data):
        if target_case in row:
            case_row = True
        if case_row:
            if keyword in row:
                processing_row = True
            if processing_row:
                if len(row) == 0:
                    break
                if (
                    floor == row[0]
                    and flame == row[1]
                    and jiku1 == row[2]
                    and jiku2 == row[3]
                ):
                    return float(row[12])

    # 対象データが見つからないとき
    print("指定したブレースの負担せん断力が見つかりませんでした")
    return 0.0


# --------------------------------------------------------------
# Q-δの取得
def get_q_delta(stepNo: int, output_filename: str) -> tuple[float, float]:
    """get_q_delta 関数

    設定で指定した階の負担せん断力(指定ステップ時)と層間変位(指定ステップ時)を取得します。

    Args:
        stepNo(int):結果を取得するステップ数
        output_csv_path(str):結果データCSVファイルのパス

    Returns:
        tuple[float, float]:負担せんだん力,層間変位

    """
    # 特定の列を識別するキーワード
    keyword = "<data>"

    # δ読み込みインデックス
    delta_index = 7

    is_saidai = False  # 重心位置＝False　最大＝True
    if is_saidai:
        delta_index = 8

    # CSVファイルを読み込む
    with open(output_filename, "r") as output_file:
        reader = csv.reader(output_file)
        data = list(reader)

    # Q-δ曲線データを取得する
    processing_row = False
    case_row = False
    for i, row in enumerate(data):
        if target_case in row:
            case_row = True
        if case_row:
            if keyword in row:
                processing_row = True
            if processing_row:
                if row and floor == row[0] and str(stepNo) == row[1]:
                    return (float(row[2]), float(row[delta_index]))

    # 対象データが見つからないとき
    return 0.0, 0.0


# --------------------------------------------------------------
# メイン
def main() -> None:
    ss7cmd.Init()
    ss7cmd.Start(None, 1)  # 1=ログファイルをクリアする

    q_list = []  # 負担せん断力を格納するリスト
    q_delta = []  # Q-δを格納するリスト

    for step in range(1, max_stepno + 1, step_pitch):
        # CSVファイルを作成する
        csv_path = output_folder + r"\output_" + str(step) + ".csv"
        make_csvfile(step, input_csv, csv_path)

        # CSV入力データからSS7のデータを作成する
        dataname = output_folder + r"\Sample1_" + str(step) + "Step.ikn"
        ss7cmd.CreateDataCsv(csv_path, dataname, 2)  # 2=中断する

        # エラー情報を取得する
        err = ss7cmd.GetLastError()

        # CSV新規作成できたことの判定
        if err.IsOK() is True:
            print("CSV新規作成が成功しました")

            # データを開く
            data = ss7cmd.Open(dataname, 1, 3)  # 1=変換して開く 3=物件データは開かない

            # データが正しく開けたことの判定
            if data is not None:

                # 計算実行
                data.Calculate("結果1", "応力解析（二次）")

                # エラー情報を取得する
                err = ss7cmd.GetLastError()

                # 計算が正しく実行できたことの判定
                if err.IsOK() is True:
                    print("計算が成功しました")

                    # 計算結果データを取得する
                    res = data.GetResultData("結果1")

                    # 結果CSVをエクスポートする
                    res.ExportResultCsv(
                        "12.9.応力表（鉛直ブレース）",
                        dataname + "\\Result1.csv",
                        2,
                        1,
                        1,
                    )  # 2=中断する 1=省略する 1=全部材出力

                    err = ss7cmd.GetLastError()
                    # 結果CSVのエクスポートが正しく実行できたことの判定
                    if err.IsOK() is True:
                        print("結果CSVのエクスポートが成功しました")
                        # ブレースの負担せん断力を取得する
                        q_list.append(get_brace_stress(dataname + "\\Result1.csv"))
                    else:
                        print("結果CSVのエクスポートが失敗しました")

                    # 結果CSVをエクスポートする
                    res.ExportResultCsv(
                        "12.24.Q-δ", dataname + "\\Result2.csv", 2, 1, 1
                    )  # 2=中断する 1=省略する 1=全部材出力

                    err = ss7cmd.GetLastError()
                    # 結果CSVのエクスポートが正しく実行できたことの判定
                    if err.IsOK() is True:
                        print("結果CSV（Q-δ曲線）のエクスポートが成功しました")
                        # Q-δ曲線取得する
                        q_delta.append(get_q_delta(step, dataname + "\\Result2.csv"))
                    else:
                        print("結果CSV（Q-δ曲線）のエクスポートが失敗しました")

                    # データを閉じる
                    data.Close(1)  # 1=保存する

                else:
                    print("計算が失敗しました")

                    # データを閉じる
                    data.Close(2)  # 2=保存しない
        else:
            print("CSV新規作成が失敗しました")

    # すべて終わったので終了する
    ss7cmd.End(1)  # 1=保存して終了

    # ----------------------------------------------------------------
    # Q-δ曲線を描画

    q_delta.insert(0, (0.0, 0.0))  # 原点を追加
    q_list.insert(0, 0)  # 原点を追加
    d_brase_values = [float(q_d[1]) for q_d in q_delta]
    q_brase_values = [float(q) for q in q_list]

    # ブレース負担せん断力曲線を描画
    plt.scatter(d_brase_values, q_brase_values)
    plt.plot(d_brase_values, q_brase_values, label="Brace")

    # グラフのタイトルと軸ラベルを設定
    plt.title("Q-Delta")
    plt.xlabel("Delta")
    plt.ylabel("Q")

    # Q-δ曲線を描画
    d_values = [float(q_d2[1]) for q_d2 in q_delta]
    q_values = [float(q_d2[0]) for q_d2 in q_delta]
    plt.scatter(d_values, q_values)
    plt.plot(d_values, q_values, label="Total")

    # 凡例を表示
    plt.legend()

    # グラフを画像で保存
    plt.savefig((output_folder + r"\ShearStressSeparation.jpg"), format="jpeg")

    # グラフを表示
    plt.show()


# ----------------------------------------------------------------
# 実行
main()
